using System;
using System.Collections;
using System.Windows.Forms;

namespace Undo_Example {
	public class CommandManager {
		private Stack undoStack;
		private Stack redoStack;
		private MenuItem undoMenu;
		private MenuItem redoMenu;

		public CommandManager(MenuItem undoMenu, MenuItem redoMenu) {
			undoStack = new Stack();
			redoStack = new Stack();
			this.undoMenu = undoMenu;
			this.redoMenu = redoMenu;

			UpdateMenuState();
		}

		public void Execute(Command cmd) {
			cmd.Execute();
			undoStack.Push(cmd);
			redoStack.Clear();

			UpdateMenuState();
		}

		public void Undo() {
			if (undoStack.Count > 0) {
				Command cmd = (Command)undoStack.Pop();
				cmd.Undo();
				redoStack.Push(cmd);
			}

			UpdateMenuState();
		}

		public void Redo() {
			if (redoStack.Count > 0) {
				Command cmd = (Command)redoStack.Pop();
				cmd.Execute();
				undoStack.Push(cmd);
			}

			UpdateMenuState();
		}

		private void UpdateMenuState() {
			if (undoStack.Count > 0) {
				undoMenu.Enabled = true;
			} else {
				undoMenu.Enabled = false;
			}

			if (redoStack.Count > 0) {
				redoMenu.Enabled = true;
			} else {
				redoMenu.Enabled = false;
			}
		}
	}
}