using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

/*
 * This code is an example of how to implement generic Undo / Redo following
 * the Command design pattern.  It uses a generic Command abstract and a 
 * CommandManager that works with any Commands.  The specific application
 * lets users add a buttons and checkboxes to a form.
 * 
 * Written by Ben Bederson, University of Maryland
 * March 7, 2005
 */
namespace Undo_Example {
	public class UndoTesterForm : System.Windows.Forms.Form {
		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem menuExit;
		private System.Windows.Forms.MenuItem menuItem3;
		private System.Windows.Forms.MenuItem menuUndo;
		private System.Windows.Forms.MenuItem menuRedo;
		private System.Windows.Forms.MenuItem menuAddButton;
		private System.Windows.Forms.MenuItem menuCheckbox;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.ComponentModel.Container components = null;

		private CommandManager cmdMgr;
		private int buttonCounter = 1;

		public UndoTesterForm() {
			InitializeComponent();

			cmdMgr = new CommandManager(menuUndo, menuRedo);
		}

		protected override void Dispose( bool disposing ) {
			if( disposing ) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.menuExit = new System.Windows.Forms.MenuItem();
			this.menuItem3 = new System.Windows.Forms.MenuItem();
			this.menuUndo = new System.Windows.Forms.MenuItem();
			this.menuRedo = new System.Windows.Forms.MenuItem();
			this.menuAddButton = new System.Windows.Forms.MenuItem();
			this.menuCheckbox = new System.Windows.Forms.MenuItem();
			this.menuItem2 = new System.Windows.Forms.MenuItem();
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuItem1,
																					  this.menuItem3});
			// 
			// menuItem1
			// 
			this.menuItem1.Index = 0;
			this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuExit});
			this.menuItem1.Text = "File";
			// 
			// menuExit
			// 
			this.menuExit.Index = 0;
			this.menuExit.Text = "Exit";
			this.menuExit.Click += new System.EventHandler(this.menuExit_Click);
			// 
			// menuItem3
			// 
			this.menuItem3.Index = 1;
			this.menuItem3.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuAddButton,
																					  this.menuCheckbox,
																					  this.menuItem2,
																					  this.menuUndo,
																					  this.menuRedo});
			this.menuItem3.Text = "Edit";
			// 
			// menuUndo
			// 
			this.menuUndo.Index = 3;
			this.menuUndo.Shortcut = System.Windows.Forms.Shortcut.CtrlZ;
			this.menuUndo.Text = "Undo";
			this.menuUndo.Click += new System.EventHandler(this.menuUndo_Click);
			// 
			// menuRedo
			// 
			this.menuRedo.Index = 4;
			this.menuRedo.Shortcut = System.Windows.Forms.Shortcut.CtrlY;
			this.menuRedo.Text = "Redo";
			this.menuRedo.Click += new System.EventHandler(this.menuRedo_Click);
			// 
			// menuAddButton
			// 
			this.menuAddButton.Index = 0;
			this.menuAddButton.Text = "Add Button";
			this.menuAddButton.Click += new System.EventHandler(this.menuAddButton_Click);
			// 
			// menuCheckbox
			// 
			this.menuCheckbox.Index = 1;
			this.menuCheckbox.Text = "Add Checkbox";
			this.menuCheckbox.Click += new System.EventHandler(this.menuCheckbox_Click);
			// 
			// menuItem2
			// 
			this.menuItem2.Index = 2;
			this.menuItem2.Text = "-";
			// 
			// UndoTesterForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(292, 266);
			this.Menu = this.mainMenu1;
			this.Name = "UndoTesterForm";
			this.Text = "Undo Tester";

		}
		#endregion

		[STAThread]
		static void Main() {
			Application.EnableVisualStyles();
			Application.Run(new UndoTesterForm());
		}

		private void menuExit_Click(object sender, System.EventArgs e) {
			Application.Exit();
		}

		private void menuAddButton_Click(object sender, System.EventArgs e) {
			Command cmd = new AddButtonCommand(this, "Button " + buttonCounter++);
			cmdMgr.Execute(cmd);
		}

		private void menuCheckbox_Click(object sender, System.EventArgs e) {
			Command cmd = new AddCheckBoxCommand(this, "Checkbox " + buttonCounter++);
			cmdMgr.Execute(cmd);		
		}

		private void menuUndo_Click(object sender, System.EventArgs e) {
			cmdMgr.Undo();
		}

		private void menuRedo_Click(object sender, System.EventArgs e) {
			cmdMgr.Redo();
		}
	}
}